/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * @Author: Fernando Arreola
 */
package org.apache.tika.parser.font;

import java.io.IOException;
import java.io.InputStream;
import java.util.Collections;
import java.util.Set;
import java.util.Iterator;
import java.util.List;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.util.regex.*;

import org.apache.fontbox.afm.AFMParser;
import org.apache.fontbox.afm.FontMetric;
import org.apache.tika.exception.TikaException;
import org.apache.tika.metadata.Property;
import org.apache.tika.metadata.Metadata;
import org.apache.tika.mime.MediaType;
import org.apache.tika.parser.ParseContext;
import org.apache.tika.parser.Parser;
import org.apache.tika.sax.XHTMLContentHandler;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

public class AdobeFontMetricParser implements Parser 
{

        private static final MediaType TYPE =
                             MediaType.application( "x-font-adobe-metric" );

        private static final Set<MediaType> SUPPORTED_TYPES = Collections.singleton(TYPE);
        
        public Set<MediaType> getSupportedTypes( ParseContext context ) 
        {
                return SUPPORTED_TYPES;
        }

        public void parse(
                        InputStream stream, ContentHandler handler,
                        Metadata metadata, ParseContext context)
                        throws IOException, SAXException, TikaException 
        {

	        FontMetric fontMetrics;
		AFMParser  parser      = new AFMParser( stream );

		// Parse the file and obtain the results
		parser.parse();
		fontMetrics = parser.getResult();

		// Get the comments in the file to display in xhtml
		List<String> comments = fontMetrics.getComments();
		
		// Get the creation date
		extractCreationDate( metadata, comments );

                metadata.set( Metadata.CONTENT_TYPE, TYPE.toString() );
		metadata.set( Metadata.TITLE, fontMetrics.getFullName() );

		// Add metadata associated with the font type
                addMetadataByString( metadata, "AvgCharacterWidth", Float.toString( fontMetrics.getAverageCharacterWidth() ) );
                addMetadataByString( metadata, "DocVersion", Float.toString( fontMetrics.getAFMVersion() ) );
                addMetadataByString( metadata, "FontFamilyName", fontMetrics.getFamilyName() );
                addMetadataByString( metadata, "FontVersion", fontMetrics.getFontVersion() );
                addMetadataByString( metadata, "FontFullName", fontMetrics.getFullName() );
                addMetadataByString( metadata, "FontWeight", fontMetrics.getWeight() );
                addMetadataByString( metadata, "FontUnderlineThickness", Float.toString( fontMetrics.getUnderlineThickness() ) );


                XHTMLContentHandler xhtml = new XHTMLContentHandler( handler, metadata );
                xhtml.startDocument();

		// Display the comments
                xhtml.element( "h1", "Comments" );

		Iterator iter = comments.iterator();
		while ( iter.hasNext() )
		{
		    String commentValue = (String) iter.next();
		   
		    xhtml.element( "p", commentValue );
	        }
          
                xhtml.endDocument();
        }

        /**
         * @deprecated This method will be removed in Apache Tika 1.0.
         */
        public void parse(
                        InputStream stream, ContentHandler handler, Metadata metadata)
                        throws IOException, SAXException, TikaException 
        {
                parse(stream, handler, metadata, new ParseContext());
        }

        private void addMetadataByString( Metadata metadata, String name, String value ) 
        {
	   // Add metadata if an appropriate value is passed 
           if (value != null) 
           {
               metadata.add( name, value );
           }
        }

        private void addMetadataByProperty( Metadata metadata, Property property, String value ) 
        {
	   // Add metadata if an appropriate value is passed 
           if (value != null) 
           {
               metadata.set( property, value );
           }
        }


        private void extractCreationDate( Metadata metadata, List<String> comments )
        {
	   Iterator iter = comments.iterator();
	   String   date = null;

	   while ( iter.hasNext() )
	   {
	       String value = (String) iter.next();

	       // Look for the creation date
	       if( value.matches( ".*Creation\\sDate.*" ) )
	       {
	           date = value.substring( value.indexOf( ":" ) + 2 );
	           comments.remove( value );

		   break;
	       }
	
	   }

	   // If appropriate date then store as metadata
	   if( date != null )
	   {
               addMetadataByProperty( metadata, Metadata.CREATION_DATE, date );
	   }

       }
}
