/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.tika.parser.font;

import java.io.InputStream;

import junit.framework.TestCase;

import org.apache.tika.metadata.Metadata;
import org.apache.tika.parser.AutoDetectParser;
import org.apache.tika.parser.ParseContext;
import org.apache.tika.parser.Parser;
import org.apache.tika.sax.BodyContentHandler;
import org.xml.sax.ContentHandler;
import org.apache.tika.io.TikaInputStream;

/**
 * Test case for parsing afm files.
 */
public class AdobeFontMetricParserTest extends TestCase {

    public void testAdobeFontMetricParsing() throws Exception {
        Parser parser = new AutoDetectParser(); // Should auto-detect!
        ContentHandler handler = new BodyContentHandler();
        Metadata metadata = new Metadata();
        ParseContext context = new ParseContext();
        TikaInputStream stream = TikaInputStream.get(
                AdobeFontMetricParserTest.class.getResource(
                        "/test-documents/testWoor2.afm"));

        try {
            parser.parse(stream, handler, metadata, context);
        } finally {
            stream.close();
        }

        assertEquals("application/x-font-adobe-metric", metadata.get(Metadata.CONTENT_TYPE));
        assertEquals("Adobe Wood Type Ornaments Two", metadata.get(Metadata.TITLE));
        assertEquals("Fri Sep 13 15:42:07 1991", metadata.get(Metadata.CREATION_DATE));

        String content = handler.toString();

	// Test that the comments got extracted
        assertTrue(content.contains("Comments"));
        assertTrue(content.contains("Copyright (c) 1990, 1991 Adobe Systems Incorporated.  All Rights Reserved."));
        assertTrue(content.contains("UniqueID 36327"));
        assertTrue(content.contains("VMusage 40580 43997"));
    }

    public void testAdobeFontMetricMetadata() throws Exception {
        Parser parser = new AutoDetectParser(); // Should auto-detect!
        ContentHandler handler = new BodyContentHandler();
        Metadata metadata = new Metadata();
        ParseContext context = new ParseContext();

        TikaInputStream stream = TikaInputStream.get(
                AdobeFontMetricParserTest.class.getResource(
                       "/test-documents/testAFM-Metadata.AFM") );

        try {
            parser.parse(stream, handler, metadata, context);
        } finally {
            stream.close();
        }

        assertEquals("application/x-font-adobe-metric", metadata.get(Metadata.CONTENT_TYPE));
        assertEquals("Minion Ornaments", metadata.get(Metadata.TITLE));
        assertEquals("Fri Jan 31 02:21:07 1992", metadata.get(Metadata.CREATION_DATE));

	// Font metadata
        assertEquals("903.2917", metadata.get("AvgCharacterWidth"));
        assertEquals("2.0", metadata.get("DocVersion"));
        assertEquals("Minion", metadata.get("FontFamilyName"));
	assertEquals("Minion Ornaments", metadata.get("FontFullName"));
        assertEquals("50.0", metadata.get("FontUnderlineThickness"));
        assertEquals("001.001", metadata.get("FontVersion"));
        assertEquals("Regular", metadata.get("FontWeight"));
        
	// Check that the creation date not duplicated in the body.
	// Even though it is a comment it should appear as metadata only
        String content = handler.toString();

        assertTrue(!content.contains("Creation Date"));
    }
  
}
